@@ -61,6 +61,27 @@ zend_class_entry *amqp_connection_class_entry;
 
 zend_object_handlers amqp_connection_object_handlers;
 
+#define PHP_AMQP_EXTRACT_CONNECTION_STR(name) \
+	zdata = NULL; \
+	if (ini_arr && PHP5to7_ZEND_HASH_FIND(HASH_OF(ini_arr), (name), sizeof(name), zdata)) { \
+		convert_to_string(PHP5to7_MAYBE_DEREF(zdata)); \
+	} \
+	if (zdata && Z_STRLEN_P(PHP5to7_MAYBE_DEREF(zdata)) > 0) { \
+		zend_update_property_string(this_ce, getThis(), ZEND_STRL(name), Z_STRVAL_P(PHP5to7_MAYBE_DEREF(zdata)) TSRMLS_CC); \
+	} else { \
+		zend_update_property_string(this_ce, getThis(), ZEND_STRL(name), INI_STR("amqp." name) TSRMLS_CC); \
+	}
+
+#define PHP_AMQP_EXTRACT_CONNECTION_BOOL(name) \
+	zdata = NULL; \
+	if (ini_arr && PHP5to7_ZEND_HASH_FIND(HASH_OF(ini_arr), (name), sizeof(name), zdata)) { \
+		convert_to_long(PHP5to7_MAYBE_DEREF(zdata)); \
+	} \
+	if (zdata) { \
+		zend_update_property_bool(this_ce, getThis(), ZEND_STRL(name), Z_LVAL_P(PHP5to7_MAYBE_DEREF(zdata)) TSRMLS_CC); \
+	} else { \
+		zend_update_property_bool(this_ce, getThis(), ZEND_STRL(name), INI_INT("amqp." name) TSRMLS_CC); \
+	}
 
 static int php_amqp_connection_resource_deleter(PHP5to7_zend_resource_le_t *el, amqp_connection_resource *connection_resource TSRMLS_DC)
 {
@@ -74,7 +95,7 @@ static int php_amqp_connection_resource_deleter(PHP5to7_zend_resource_le_t *el,
 static PHP5to7_param_str_len_type_t php_amqp_get_connection_hash(amqp_connection_params *params, char **hash) {
 	return spprintf(hash,
 					0,
-					"amqp_conn_res_h:%s_p:%d_v:%s_l:%s_p:%s_f:%d_c:%d_h:%d",
+					"amqp_conn_res_h:%s_p:%d_v:%s_l:%s_p:%s_f:%d_c:%d_h:%d_cacert:%s_cert:%s_key:%s",
 					params->host,
 					params->port,
 					params->vhost,
@@ -82,7 +103,10 @@ static PHP5to7_param_str_len_type_t php_amqp_get_connection_hash(amqp_connection
 					params->password,
 					params->frame_max,
 					params->channel_max,
-					params->heartbeat
+					params->heartbeat,
+					params->cacert,
+					params->cert,
+					params->key
 	);
 }
 
@@ -160,7 +184,10 @@ int php_amqp_connect(amqp_connection_object *connection, zend_bool persistent, I
 	connection_params.read_timeout = PHP_AMQP_READ_THIS_PROP_DOUBLE("read_timeout");
 	connection_params.write_timeout = PHP_AMQP_READ_THIS_PROP_DOUBLE("write_timeout");
 	connection_params.connect_timeout = PHP_AMQP_READ_THIS_PROP_DOUBLE("connect_timeout");
-
+	connection_params.cacert = PHP_AMQP_READ_THIS_PROP_STRLEN("cacert") ? PHP_AMQP_READ_THIS_PROP_STR("cacert") : NULL;
+	connection_params.cert = PHP_AMQP_READ_THIS_PROP_STRLEN("cert") ? PHP_AMQP_READ_THIS_PROP_STR("cert") : NULL;
+	connection_params.key = PHP_AMQP_READ_THIS_PROP_STRLEN("key") ? PHP_AMQP_READ_THIS_PROP_STR("key") : NULL;
+	connection_params.verify = (int) PHP_AMQP_READ_THIS_PROP_BOOL("verify");
 
 	if (persistent) {
 		PHP5to7_zend_resource_store_t *le = PHP5to7_ZEND_RESOURCE_EMPTY;
@@ -289,7 +316,7 @@ PHP5to7_zend_object_value amqp_connection_ctor(zend_class_entry *ce TSRMLS_DC)
 /* {{{ proto AMQPConnection::__construct([array optional])
  * The array can contain 'host', 'port', 'login', 'password', 'vhost', 'read_timeout', 'write_timeout', 'connect_timeout' and 'timeout' (deprecated) indexes
  */
-PHP_METHOD(amqp_connection_class, __construct)
+static PHP_METHOD(amqp_connection_class, __construct)
 {
 	zval* ini_arr = NULL;
 
@@ -483,13 +510,18 @@ PHP_METHOD(amqp_connection_class, __construct)
 		}
 	}
 
+	PHP_AMQP_EXTRACT_CONNECTION_STR("cacert");
+	PHP_AMQP_EXTRACT_CONNECTION_STR("key");
+	PHP_AMQP_EXTRACT_CONNECTION_STR("cert");
+
+	PHP_AMQP_EXTRACT_CONNECTION_BOOL("verify");
 }
 /* }}} */
 
 
 /* {{{ proto amqp::isConnected()
 check amqp connection */
-PHP_METHOD(amqp_connection_class, isConnected)
+static PHP_METHOD(amqp_connection_class, isConnected)
 {
 	amqp_connection_object *connection;
 
@@ -511,7 +543,7 @@ PHP_METHOD(amqp_connection_class, isConnected)
 
 /* {{{ proto amqp::connect()
 create amqp connection */
-PHP_METHOD(amqp_connection_class, connect)
+static PHP_METHOD(amqp_connection_class, connect)
 {
 	amqp_connection_object *connection;
 
@@ -536,7 +568,7 @@ PHP_METHOD(amqp_connection_class, connect)
 
 /* {{{ proto amqp::connect()
 create amqp connection */
-PHP_METHOD(amqp_connection_class, pconnect)
+static PHP_METHOD(amqp_connection_class, pconnect)
 {
 	amqp_connection_object *connection;
 
@@ -563,7 +595,7 @@ PHP_METHOD(amqp_connection_class, pconnect)
 
 /* {{{ proto amqp:pdisconnect()
 destroy amqp persistent connection */
-PHP_METHOD(amqp_connection_class, pdisconnect)
+static PHP_METHOD(amqp_connection_class, pdisconnect)
 {
 	amqp_connection_object *connection;
 
@@ -593,7 +625,7 @@ PHP_METHOD(amqp_connection_class, pdisconnect)
 
 /* {{{ proto amqp::disconnect()
 destroy amqp connection */
-PHP_METHOD(amqp_connection_class, disconnect)
+static PHP_METHOD(amqp_connection_class, disconnect)
 {
 	amqp_connection_object *connection;
 
@@ -623,7 +655,7 @@ PHP_METHOD(amqp_connection_class, disconnect)
 
 /* {{{ proto amqp::reconnect()
 recreate amqp connection */
-PHP_METHOD(amqp_connection_class, reconnect)
+static PHP_METHOD(amqp_connection_class, reconnect)
 {
 	amqp_connection_object *connection;
 
@@ -651,7 +683,7 @@ PHP_METHOD(amqp_connection_class, reconnect)
 
 /* {{{ proto amqp::preconnect()
 recreate amqp connection */
-PHP_METHOD(amqp_connection_class, preconnect)
+static PHP_METHOD(amqp_connection_class, preconnect)
 {
 	amqp_connection_object *connection;
 
@@ -681,7 +713,7 @@ PHP_METHOD(amqp_connection_class, preconnect)
 
 /* {{{ proto amqp::getLogin()
 get the login */
-PHP_METHOD(amqp_connection_class, getLogin)
+static PHP_METHOD(amqp_connection_class, getLogin)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -692,7 +724,7 @@ PHP_METHOD(amqp_connection_class, getLogin)
 
 /* {{{ proto amqp::setLogin(string login)
 set the login */
-PHP_METHOD(amqp_connection_class, setLogin)
+static PHP_METHOD(amqp_connection_class, setLogin)
 {
 	char *login = NULL;	PHP5to7_param_str_len_type_t login_len = 0;
 
@@ -715,7 +747,7 @@ PHP_METHOD(amqp_connection_class, setLogin)
 
 /* {{{ proto amqp::getPassword()
 get the password */
-PHP_METHOD(amqp_connection_class, getPassword)
+static PHP_METHOD(amqp_connection_class, getPassword)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -726,7 +758,7 @@ PHP_METHOD(amqp_connection_class, getPassword)
 
 /* {{{ proto amqp::setPassword(string password)
 set the password */
-PHP_METHOD(amqp_connection_class, setPassword)
+static PHP_METHOD(amqp_connection_class, setPassword)
 {
 	char *password = NULL;	PHP5to7_param_str_len_type_t password_len = 0;
 
@@ -750,7 +782,7 @@ PHP_METHOD(amqp_connection_class, setPassword)
 
 /* {{{ proto amqp::getHost()
 get the host */
-PHP_METHOD(amqp_connection_class, getHost)
+static PHP_METHOD(amqp_connection_class, getHost)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -761,7 +793,7 @@ PHP_METHOD(amqp_connection_class, getHost)
 
 /* {{{ proto amqp::setHost(string host)
 set the host */
-PHP_METHOD(amqp_connection_class, setHost)
+static PHP_METHOD(amqp_connection_class, setHost)
 {
 	char *host = NULL;	PHP5to7_param_str_len_type_t host_len = 0;
 
@@ -785,7 +817,7 @@ PHP_METHOD(amqp_connection_class, setHost)
 
 /* {{{ proto amqp::getPort()
 get the port */
-PHP_METHOD(amqp_connection_class, getPort)
+static PHP_METHOD(amqp_connection_class, getPort)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -796,7 +828,7 @@ PHP_METHOD(amqp_connection_class, getPort)
 
 /* {{{ proto amqp::setPort(mixed port)
 set the port */
-PHP_METHOD(amqp_connection_class, setPort)
+static PHP_METHOD(amqp_connection_class, setPort)
 {
 	zval *zvalPort;
 	int port;
@@ -836,7 +868,7 @@ PHP_METHOD(amqp_connection_class, setPort)
 
 /* {{{ proto amqp::getVhost()
 get the vhost */
-PHP_METHOD(amqp_connection_class, getVhost)
+static PHP_METHOD(amqp_connection_class, getVhost)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -847,11 +879,10 @@ PHP_METHOD(amqp_connection_class, getVhost)
 
 /* {{{ proto amqp::setVhost(string vhost)
 set the vhost */
-PHP_METHOD(amqp_connection_class, setVhost)
+static PHP_METHOD(amqp_connection_class, setVhost)
 {
 	char *vhost = NULL;	PHP5to7_param_str_len_type_t vhost_len = 0;
 
-	/* Get the vhost from the method params */
 	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &vhost, &vhost_len) == FAILURE) {
 		return;
 	}
@@ -871,7 +902,7 @@ PHP_METHOD(amqp_connection_class, setVhost)
 /* {{{ proto amqp::getTimeout()
 @deprecated
 get the timeout */
-PHP_METHOD(amqp_connection_class, getTimeout)
+static PHP_METHOD(amqp_connection_class, getTimeout)
 {
 	php_error_docref(NULL TSRMLS_CC, E_DEPRECATED, "AMQPConnection::getTimeout() method is deprecated; use AMQPConnection::getReadTimeout() instead");
 
@@ -884,7 +915,7 @@ PHP_METHOD(amqp_connection_class, getTimeout)
 /* {{{ proto amqp::setTimeout(double timeout)
 @deprecated
 set the timeout */
-PHP_METHOD(amqp_connection_class, setTimeout)
+static PHP_METHOD(amqp_connection_class, setTimeout)
 {
 	amqp_connection_object *connection;
 	double read_timeout;
@@ -922,7 +953,7 @@ PHP_METHOD(amqp_connection_class, setTimeout)
 
 /* {{{ proto amqp::getReadTimeout()
 get the read timeout */
-PHP_METHOD(amqp_connection_class, getReadTimeout)
+static PHP_METHOD(amqp_connection_class, getReadTimeout)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -932,7 +963,7 @@ PHP_METHOD(amqp_connection_class, getReadTimeout)
 
 /* {{{ proto amqp::setReadTimeout(double timeout)
 set read timeout */
-PHP_METHOD(amqp_connection_class, setReadTimeout)
+static PHP_METHOD(amqp_connection_class, setReadTimeout)
 {
 	amqp_connection_object *connection;
 	double read_timeout;
@@ -968,7 +999,7 @@ PHP_METHOD(amqp_connection_class, setReadTimeout)
 
 /* {{{ proto amqp::getWriteTimeout()
 get write timeout */
-PHP_METHOD(amqp_connection_class, getWriteTimeout)
+static PHP_METHOD(amqp_connection_class, getWriteTimeout)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -978,7 +1009,7 @@ PHP_METHOD(amqp_connection_class, getWriteTimeout)
 
 /* {{{ proto amqp::setWriteTimeout(double timeout)
 set write timeout */
-PHP_METHOD(amqp_connection_class, setWriteTimeout)
+static PHP_METHOD(amqp_connection_class, setWriteTimeout)
 {
 	amqp_connection_object *connection;
 	double write_timeout;
@@ -1014,7 +1045,7 @@ PHP_METHOD(amqp_connection_class, setWriteTimeout)
 
 /* {{{ proto amqp::getUsedChannels()
 Get max used channels number */
-PHP_METHOD(amqp_connection_class, getUsedChannels)
+static PHP_METHOD(amqp_connection_class, getUsedChannels)
 {
 	amqp_connection_object *connection;
 
@@ -1056,7 +1087,7 @@ PHP_METHOD(amqp_connection_class, getMaxChannels)
 
 /* {{{ proto amqp::getMaxFrameSize()
 Get max supported frame size per connection in bytes */
-PHP_METHOD(amqp_connection_class, getMaxFrameSize)
+static PHP_METHOD(amqp_connection_class, getMaxFrameSize)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	amqp_connection_object *connection;
@@ -1078,7 +1109,7 @@ PHP_METHOD(amqp_connection_class, getMaxFrameSize)
 
 /* {{{ proto amqp::getHeartbeatInterval()
 Get number of seconds between heartbeats of the connection in seconds */
-PHP_METHOD(amqp_connection_class, getHeartbeatInterval)
+static PHP_METHOD(amqp_connection_class, getHeartbeatInterval)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	amqp_connection_object *connection;
@@ -1101,7 +1132,7 @@ PHP_METHOD(amqp_connection_class, getHeartbeatInterval)
 
 /* {{{ proto amqp::isPersistent()
 check whether amqp connection is persistent */
-PHP_METHOD(amqp_connection_class, isPersistent)
+static PHP_METHOD(amqp_connection_class, isPersistent)
 {
 	amqp_connection_object *connection;
 
@@ -1114,6 +1145,103 @@ PHP_METHOD(amqp_connection_class, isPersistent)
 /* }}} */
 
 
+/* {{{ proto amqp::getCACert() */
+static PHP_METHOD(amqp_connection_class, getCACert)
+{
+	PHP5to7_READ_PROP_RV_PARAM_DECL;
+	PHP_AMQP_NOPARAMS();
+	PHP_AMQP_RETURN_THIS_PROP("cacert");
+}
+/* }}} */
+
+/* {{{ proto amqp::setCACert(string cacert) */
+static PHP_METHOD(amqp_connection_class, setCACert)
+{
+	char *str = NULL;	PHP5to7_param_str_len_type_t str_len = 0;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) == FAILURE) {
+		return;
+	}
+
+	zend_update_property_stringl(this_ce, getThis(), ZEND_STRL("cacert"), str, str_len TSRMLS_CC);
+
+	RETURN_TRUE;
+}
+/* }}} */
+
+/* {{{ proto amqp::getCert() */
+static PHP_METHOD(amqp_connection_class, getCert)
+{
+	PHP5to7_READ_PROP_RV_PARAM_DECL;
+	PHP_AMQP_NOPARAMS();
+	PHP_AMQP_RETURN_THIS_PROP("cert");
+}
+/* }}} */
+
+/* {{{ proto amqp::setCert(string cert) */
+static PHP_METHOD(amqp_connection_class, setCert)
+{
+	char *str = NULL;	PHP5to7_param_str_len_type_t str_len = 0;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) == FAILURE) {
+		return;
+	}
+
+	zend_update_property_stringl(this_ce, getThis(), ZEND_STRL("cert"), str, str_len TSRMLS_CC);
+
+	RETURN_TRUE;
+}
+/* }}} */
+
+/* {{{ proto amqp::getKey() */
+static PHP_METHOD(amqp_connection_class, getKey)
+{
+	PHP5to7_READ_PROP_RV_PARAM_DECL;
+	PHP_AMQP_NOPARAMS();
+	PHP_AMQP_RETURN_THIS_PROP("key");
+}
+/* }}} */
+
+/* {{{ proto amqp::setKey(string key) */
+static PHP_METHOD(amqp_connection_class, setKey)
+{
+	char *str = NULL;	PHP5to7_param_str_len_type_t str_len = 0;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &str, &str_len) == FAILURE) {
+		return;
+	}
+
+	zend_update_property_stringl(this_ce, getThis(), ZEND_STRL("key"), str, str_len TSRMLS_CC);
+
+	RETURN_TRUE;
+}
+/* }}} */
+
+
+/* {{{ proto amqp::getVerify() */
+static PHP_METHOD(amqp_connection_class, getVerify)
+{
+	PHP5to7_READ_PROP_RV_PARAM_DECL;
+	PHP_AMQP_NOPARAMS();
+	PHP_AMQP_RETURN_THIS_PROP("verify");
+}
+/* }}} */
+
+/* {{{ proto amqp::setVerify(bool verify) */
+static PHP_METHOD(amqp_connection_class, setVerify)
+{
+	zend_bool verify = 1;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "b", &verify) == FAILURE) {
+		return;
+	}
+
+	zend_update_property_bool(this_ce, getThis(), ZEND_STRL("verify"), verify TSRMLS_CC);
+
+	RETURN_TRUE;
+}
+/* }}} */
+
 /* amqp_connection_class ARG_INFO definition */
 ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class__construct, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
 				ZEND_ARG_ARRAY_INFO(0, credentials, 0)
@@ -1211,6 +1339,34 @@ ZEND_END_ARG_INFO()
 ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_isPersistent, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
 ZEND_END_ARG_INFO()
 
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_getCACert, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_setCACert, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
+				ZEND_ARG_INFO(0, cacert)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_getCert, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_setCert, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
+				ZEND_ARG_INFO(0, cert)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_getKey, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_setKey, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
+				ZEND_ARG_INFO(0, key)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_getVerify, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 0)
+ZEND_END_ARG_INFO()
+
+ZEND_BEGIN_ARG_INFO_EX(arginfo_amqp_connection_class_setVerify, ZEND_SEND_BY_VAL, ZEND_RETURN_VALUE, 1)
+				ZEND_ARG_INFO(0, verify)
+ZEND_END_ARG_INFO()
+
 
 zend_function_entry amqp_connection_class_functions[] = {
 		PHP_ME(amqp_connection_class, __construct, 	arginfo_amqp_connection_class__construct,	ZEND_ACC_PUBLIC)
@@ -1252,7 +1408,19 @@ zend_function_entry amqp_connection_class_functions[] = {
 		PHP_ME(amqp_connection_class, getHeartbeatInterval,  arginfo_amqp_connection_class_getHeartbeatInterval,	ZEND_ACC_PUBLIC)
 		PHP_ME(amqp_connection_class, getMaxFrameSize,  arginfo_amqp_connection_class_getMaxFrameSize,	ZEND_ACC_PUBLIC)
 
-		{NULL, NULL, NULL}	/* Must be the last line in amqp_functions[] */
+		PHP_ME(amqp_connection_class, getCACert, 	arginfo_amqp_connection_class_getCACert,		ZEND_ACC_PUBLIC)
+		PHP_ME(amqp_connection_class, setCACert, 	arginfo_amqp_connection_class_setCACert,		ZEND_ACC_PUBLIC)
+
+		PHP_ME(amqp_connection_class, getCert, 	arginfo_amqp_connection_class_getCert,		ZEND_ACC_PUBLIC)
+		PHP_ME(amqp_connection_class, setCert, 	arginfo_amqp_connection_class_setCert,		ZEND_ACC_PUBLIC)
+
+		PHP_ME(amqp_connection_class, getKey, 	arginfo_amqp_connection_class_getKey,		ZEND_ACC_PUBLIC)
+		PHP_ME(amqp_connection_class, setKey, 	arginfo_amqp_connection_class_setKey,		ZEND_ACC_PUBLIC)
+
+		PHP_ME(amqp_connection_class, getVerify, 	arginfo_amqp_connection_class_getVerify,		ZEND_ACC_PUBLIC)
+		PHP_ME(amqp_connection_class, setVerify, 	arginfo_amqp_connection_class_setVerify,		ZEND_ACC_PUBLIC)
+
+		{NULL, NULL, NULL}
 };
 
 
@@ -1278,6 +1446,11 @@ PHP_MINIT_FUNCTION(amqp_connection)
 	zend_declare_property_null(this_ce, ZEND_STRL("frame_max"), ZEND_ACC_PRIVATE TSRMLS_CC);
 	zend_declare_property_null(this_ce, ZEND_STRL("heartbeat"), ZEND_ACC_PRIVATE TSRMLS_CC);
 
+	zend_declare_property_null(this_ce, ZEND_STRL("cacert"), ZEND_ACC_PRIVATE TSRMLS_CC);
+	zend_declare_property_null(this_ce, ZEND_STRL("key"), ZEND_ACC_PRIVATE TSRMLS_CC);
+	zend_declare_property_null(this_ce, ZEND_STRL("cert"), ZEND_ACC_PRIVATE TSRMLS_CC);
+	zend_declare_property_null(this_ce, ZEND_STRL("verify"), ZEND_ACC_PRIVATE TSRMLS_CC);
+
 #if PHP_MAJOR_VERSION >=7
 	memcpy(&amqp_connection_object_handlers, zend_get_std_object_handlers(), sizeof(zend_object_handlers));
 
