@@ -20,9 +20,6 @@
   | - Jonathan Tansavatdi                                                |
   +----------------------------------------------------------------------+
 */
-
-/* $Id: amqp_queue.c 327551 2012-09-09 03:49:34Z pdezwart $ */
-
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -53,6 +50,7 @@
 #include "amqp_connection.h"
 #include "amqp_channel.h"
 #include "amqp_queue.h"
+#include "amqp_type.h"
 
 zend_class_entry *amqp_queue_class_entry;
 #define this_ce amqp_queue_class_entry
@@ -61,7 +59,7 @@ zend_class_entry *amqp_queue_class_entry;
 /* {{{ proto AMQPQueue::__construct(AMQPChannel channel)
 AMQPQueue constructor
 */
-PHP_METHOD(amqp_queue_class, __construct)
+static PHP_METHOD(amqp_queue_class, __construct)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -77,7 +75,7 @@ PHP_METHOD(amqp_queue_class, __construct)
 	PHP5to7_MAYBE_INIT(arguments);
 	PHP5to7_ARRAY_INIT(arguments);
 	zend_update_property(this_ce, getThis(), ZEND_STRL("arguments"), PHP5to7_MAYBE_PTR(arguments) TSRMLS_CC);
-    PHP5to7_MAYBE_DESTROY(arguments);
+	PHP5to7_MAYBE_DESTROY(arguments);
 
 	channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(channelObj);
 	PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not create queue.");
@@ -91,7 +89,7 @@ PHP_METHOD(amqp_queue_class, __construct)
 
 /* {{{ proto AMQPQueue::getName()
 Get the queue name */
-PHP_METHOD(amqp_queue_class, getName)
+static PHP_METHOD(amqp_queue_class, getName)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -109,7 +107,7 @@ PHP_METHOD(amqp_queue_class, getName)
 
 /* {{{ proto AMQPQueue::setName(string name)
 Set the queue name */
-PHP_METHOD(amqp_queue_class, setName)
+static PHP_METHOD(amqp_queue_class, setName)
 {
 	char *name = NULL;	PHP5to7_param_str_len_type_t name_len = 0;
 
@@ -135,7 +133,7 @@ PHP_METHOD(amqp_queue_class, setName)
 
 /* {{{ proto AMQPQueue::getFlags()
 Get the queue parameters */
-PHP_METHOD(amqp_queue_class, getFlags)
+static PHP_METHOD(amqp_queue_class, getFlags)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -166,7 +164,7 @@ PHP_METHOD(amqp_queue_class, getFlags)
 
 /* {{{ proto AMQPQueue::setFlags(long bitmask)
 Set the queue parameters */
-PHP_METHOD(amqp_queue_class, setFlags)
+static PHP_METHOD(amqp_queue_class, setFlags)
 {
 	PHP5to7_param_long_type_t flagBitmask;
 
@@ -190,7 +188,7 @@ PHP_METHOD(amqp_queue_class, setFlags)
 
 /* {{{ proto AMQPQueue::getArgument(string key)
 Get the queue argument referenced by key */
-PHP_METHOD(amqp_queue_class, getArgument)
+static PHP_METHOD(amqp_queue_class, getArgument)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -211,7 +209,7 @@ PHP_METHOD(amqp_queue_class, getArgument)
 /* }}} */
 
 /* {{{ proto AMQPQueue::hasArgument(string key) */
-PHP_METHOD(amqp_queue_class, hasArgument)
+static PHP_METHOD(amqp_queue_class, hasArgument)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -234,7 +232,7 @@ PHP_METHOD(amqp_queue_class, hasArgument)
 
 /* {{{ proto AMQPQueue::getArguments
 Get the queue arguments */
-PHP_METHOD(amqp_queue_class, getArguments)
+static PHP_METHOD(amqp_queue_class, getArguments)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -244,11 +242,11 @@ PHP_METHOD(amqp_queue_class, getArguments)
 
 /* {{{ proto AMQPQueue::setArguments(array args)
 Overwrite all queue arguments with given args */
-PHP_METHOD(amqp_queue_class, setArguments)
+static PHP_METHOD(amqp_queue_class, setArguments)
 {
 	zval *zvalArguments;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a", &zvalArguments) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "a/", &zvalArguments) == FAILURE) {
 		return;
 	}
 
@@ -261,7 +259,7 @@ PHP_METHOD(amqp_queue_class, setArguments)
 
 /* {{{ proto AMQPQueue::setArgument(key, value)
 Get the queue name */
-PHP_METHOD(amqp_queue_class, setArgument)
+static PHP_METHOD(amqp_queue_class, setArgument)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -298,7 +296,7 @@ PHP_METHOD(amqp_queue_class, setArgument)
 /* {{{ proto int AMQPQueue::declareQueue();
 declare queue
 */
-PHP_METHOD(amqp_queue_class, declareQueue)
+static PHP_METHOD(amqp_queue_class, declareQueue)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -315,7 +313,7 @@ PHP_METHOD(amqp_queue_class, declareQueue)
 	channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
 	PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not declare queue.");
 
-	arguments = convert_zval_to_amqp_table(PHP_AMQP_READ_THIS_PROP("arguments") TSRMLS_CC);
+	arguments = php_amqp_type_convert_zval_to_amqp_table(PHP_AMQP_READ_THIS_PROP("arguments") TSRMLS_CC);
 
 	amqp_queue_declare_ok_t *r = amqp_queue_declare(
 		channel_resource->connection_resource->connection_state,
@@ -328,26 +326,22 @@ PHP_METHOD(amqp_queue_class, declareQueue)
 		*arguments
 	);
 
-	php_amqp_free_amqp_table(arguments);
+	php_amqp_type_free_amqp_table(arguments);
 
 	if (!r) {
 		amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
 	message_count = r->message_count;
 
 	/* Set the queue name, in case it is an autogenerated queue name */
-	name = stringify_bytes(r->queue);
+	name = php_amqp_type_amqp_bytes_to_char(r->queue);
 	zend_update_property_string(this_ce, getThis(), ZEND_STRL("name"), name TSRMLS_CC);
 	efree(name);
 
@@ -361,7 +355,7 @@ PHP_METHOD(amqp_queue_class, declareQueue)
 /* {{{ proto int AMQPQueue::bind(string exchangeName, [string routingKey, array arguments]);
 bind queue to exchange by routing key
 */
-PHP_METHOD(amqp_queue_class, bind)
+static PHP_METHOD(amqp_queue_class, bind)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -385,7 +379,7 @@ PHP_METHOD(amqp_queue_class, bind)
 	PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not bind queue.");
 
 	if (zvalArguments) {
-		arguments = convert_zval_to_amqp_table(zvalArguments TSRMLS_CC);
+		arguments = php_amqp_type_convert_zval_to_amqp_table(zvalArguments TSRMLS_CC);
 	}
 
 	amqp_queue_bind(
@@ -398,20 +392,14 @@ PHP_METHOD(amqp_queue_class, bind)
 	);
 
 	if (arguments) {
-		php_amqp_free_amqp_table(arguments);
+		php_amqp_type_free_amqp_table(arguments);
 	}
 
 	amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
-		PHP_AMQP_INIT_ERROR_MESSAGE();
-
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+	if (PHP_AMQP_MAYBE_ERROR(res, channel_resource)) {
+		php_amqp_zend_throw_exception_short(res, amqp_queue_exception_class_entry TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -426,7 +414,7 @@ PHP_METHOD(amqp_queue_class, bind)
 read messages from queue
 return array (messages)
 */
-PHP_METHOD(amqp_queue_class, get)
+static PHP_METHOD(amqp_queue_class, get)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -452,19 +440,14 @@ PHP_METHOD(amqp_queue_class, get)
 		(AMQP_AUTOACK & flags) ? 1 : 0
 	);
 
-	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
-		PHP_AMQP_INIT_ERROR_MESSAGE();
-
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+	if (PHP_AMQP_MAYBE_ERROR(res, channel_resource)) {
+		php_amqp_zend_throw_exception_short(res, amqp_queue_exception_class_entry TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
 	if (AMQP_BASIC_GET_EMPTY_METHOD == res.reply.id) {
+		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
 		RETURN_FALSE;
 	}
 
@@ -491,16 +474,10 @@ PHP_METHOD(amqp_queue_class, get)
 		0
 	);
 
-	if (AMQP_RESPONSE_NORMAL != res.reply_type) {
-		PHP_AMQP_INIT_ERROR_MESSAGE();
-
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+	if (PHP_AMQP_MAYBE_ERROR(res, channel_resource)) {
+		php_amqp_zend_throw_exception_short(res, amqp_queue_exception_class_entry TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
 		amqp_destroy_envelope(&envelope);
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -520,11 +497,17 @@ PHP_METHOD(amqp_queue_class, get)
 /* {{{ proto array AMQPQueue::consume([callback, flags = <bitmask>, consumer_tag]);
 consume the message
 */
-PHP_METHOD(amqp_queue_class, consume)
+static PHP_METHOD(amqp_queue_class, consume)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
+	PHP5to7_zval_t *consumer_tag_zv = NULL;
+	PHP5to7_zval_t current_channel_zv PHP5to7_MAYBE_SET_TO_NULL;
+
+	PHP5to7_zval_t *current_queue_zv = NULL;
+
 	amqp_channel_resource *channel_resource;
+	amqp_channel_resource *current_channel_resource;
 
 	zend_fcall_info fci = empty_fcall_info;
 	zend_fcall_info_cache fci_cache = empty_fcall_info_cache;
@@ -541,12 +524,22 @@ PHP_METHOD(amqp_queue_class, consume)
 		return;
 	}
 
-	channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
+	zval *channel_zv = PHP_AMQP_READ_THIS_PROP("channel");
+	zval *consumers = zend_read_property(amqp_channel_class_entry, channel_zv, ZEND_STRL("consumers"), 0 PHP5to7_READ_PROP_RV_PARAM_CC TSRMLS_CC);
+
+	if (IS_ARRAY != Z_TYPE_P(consumers)) {
+		zend_throw_exception(amqp_queue_exception_class_entry, "Invalid channel consumers, forgot to call channel constructor?", 0 TSRMLS_CC);
+		return;
+	}
+
+	amqp_channel_object *channel = PHP_AMQP_GET_CHANNEL(channel_zv);
+
+	channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(channel_zv);
 	PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not get channel.");
 
 	if (!(AMQP_JUST_CONSUME & flags)) {
 		/* Setup the consume */
-		arguments = convert_zval_to_amqp_table(PHP_AMQP_READ_THIS_PROP("arguments") TSRMLS_CC);
+		arguments = php_amqp_type_convert_zval_to_amqp_table(PHP_AMQP_READ_THIS_PROP("arguments") TSRMLS_CC);
 
 		amqp_basic_consume_ok_t *r = amqp_basic_consume(
 				channel_resource->connection_resource->connection_state,
@@ -559,22 +552,47 @@ PHP_METHOD(amqp_queue_class, consume)
 				*arguments
 		);
 
-		php_amqp_free_amqp_table(arguments);
+		php_amqp_type_free_amqp_table(arguments);
 
 		if (!r) {
 			amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-			PHP_AMQP_INIT_ERROR_MESSAGE();
+			php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-			php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-			zend_throw_exception(amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+			zend_throw_exception(amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 			php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-			PHP_AMQP_DESTROY_ERROR_MESSAGE();
 			return;
 		}
 
+		char *key;
+		key = estrndup((char *) r->consumer_tag.bytes, (uint) r->consumer_tag.len);
+
+		if (PHP5to7_ZEND_HASH_FIND(Z_ARRVAL_P(consumers), (const char *) key, PHP5to7_ZEND_HASH_STRLEN(r->consumer_tag.len), consumer_tag_zv)) {
+			// This should never happen as AMQP server guarantees that consumer tag is unique within channel
+			zend_throw_exception(amqp_exception_class_entry, "Duplicate consumer tag", 0 TSRMLS_CC);
+		    efree(key);
+			return;
+		}
+
+		PHP5to7_zval_t tmp PHP5to7_MAYBE_SET_TO_NULL;
+
+#if PHP_MAJOR_VERSION >= 7
+		PHP5to7_MAYBE_INIT(tmp);
+		ZVAL_COPY(PHP5to7_MAYBE_PTR(tmp), getThis());
+#else
+		tmp = getThis();
+		Z_ADDREF_P(tmp);
+#endif
+
+		PHP5to7_ZEND_HASH_ADD(Z_ARRVAL_P(consumers),
+		                      (const char *) key,
+		                      PHP5to7_ZEND_HASH_STRLEN(r->consumer_tag.len),
+		                      PHP5to7_MAYBE_PTR(tmp),
+		                      sizeof(PHP5to7_MAYBE_PTR_TYPE)
+		);
+
+		efree(key);
+
 		/* Set the consumer tag name, in case it is an autogenerated consumer tag name */
 		zend_update_property_stringl(this_ce, getThis(), ZEND_STRL("consumer_tag"), (const char *) r->consumer_tag.bytes, (PHP5to7_param_str_len_type_t) r->consumer_tag.len TSRMLS_CC);
 	}
@@ -607,34 +625,83 @@ PHP_METHOD(amqp_queue_class, consume)
 		amqp_rpc_reply_t res = amqp_consume_message(channel_resource->connection_resource->connection_state, &envelope, tv_ptr, 0);
 
 		if (AMQP_RESPONSE_LIBRARY_EXCEPTION == res.reply_type && AMQP_STATUS_TIMEOUT == res.library_error) {
-			PHP_AMQP_INIT_ERROR_MESSAGE();
-
 			zend_throw_exception(amqp_queue_exception_class_entry, "Consumer timeout exceed", 0 TSRMLS_CC);
 
 			amqp_destroy_envelope(&envelope);
 			php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-			PHP_AMQP_DESTROY_ERROR_MESSAGE();
 			return;
 		}
 
-		if (AMQP_RESPONSE_NORMAL != res.reply_type) {
-			PHP_AMQP_INIT_ERROR_MESSAGE();
+		if (PHP_AMQP_MAYBE_ERROR_RECOVERABLE(res, channel_resource)) {
 
-			php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
+			if (PHP_AMQP_IS_ERROR_RECOVERABLE(res, channel_resource, channel)) {
+				/* In case no message was received, continue the loop */
+				amqp_destroy_envelope(&envelope);
 
-			php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+				continue;
+			} else {
+				/* Mark connection resource as closed to prevent sending any further requests */
+				channel_resource->connection_resource->is_connected = '\0';
+
+				/* Close connection with all its channels */
+				php_amqp_disconnect_force(channel_resource->connection_resource TSRMLS_CC);
+			}
+
+			php_amqp_zend_throw_exception_short(res, amqp_queue_exception_class_entry TSRMLS_CC);
 
 			amqp_destroy_envelope(&envelope);
 			php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
 
-			PHP_AMQP_DESTROY_ERROR_MESSAGE();
 			return;
 		}
 
 		PHP5to7_MAYBE_INIT(message);
 		convert_amqp_envelope_to_zval(&envelope, PHP5to7_MAYBE_PTR(message) TSRMLS_CC);
 
+		current_channel_resource = channel_resource->connection_resource->slots[envelope.channel - 1];
+
+		if (!current_channel_resource) {
+			// This should never happen, but just in case
+			php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, "Orphaned channel. Please, report a bug.", 0 TSRMLS_CC);
+			amqp_destroy_envelope(&envelope);
+			break;
+		}
+
+#if PHP_MAJOR_VERSION >= 7
+		PHP5to7_MAYBE_INIT(current_channel_zv);
+		ZVAL_OBJ(&current_channel_zv, &current_channel_resource->parent->zo);
+#else
+		current_channel_zv = current_channel_resource->parent->this_ptr;
+#endif
+
+		consumers = zend_read_property(amqp_channel_class_entry, PHP5to7_MAYBE_PTR(current_channel_zv), ZEND_STRL("consumers"), 0 PHP5to7_READ_PROP_RV_PARAM_CC TSRMLS_CC);
+
+		if (IS_ARRAY != Z_TYPE_P(consumers)) {
+			zend_throw_exception(amqp_queue_exception_class_entry, "Invalid channel consumers, forgot to call channel constructor?", 0 TSRMLS_CC);
+			amqp_destroy_envelope(&envelope);
+			break;
+		}
+
+		char *key;
+		key = estrndup((char *)envelope.consumer_tag.bytes, (uint) envelope.consumer_tag.len);
+
+		if (!PHP5to7_ZEND_HASH_FIND(Z_ARRVAL_P(consumers), key, PHP5to7_ZEND_HASH_STRLEN(envelope.consumer_tag.len), current_queue_zv)) {
+			PHP5to7_zval_t exception PHP5to7_MAYBE_SET_TO_NULL;
+			PHP5to7_MAYBE_INIT(exception);
+			object_init_ex(PHP5to7_MAYBE_PTR(exception), amqp_envelope_exception_class_entry);
+		    zend_update_property_string(zend_exception_get_default(TSRMLS_C), PHP5to7_MAYBE_PTR(exception),  ZEND_STRL("message"), "Orphaned envelope" TSRMLS_CC);
+			zend_update_property(amqp_envelope_exception_class_entry, PHP5to7_MAYBE_PTR(exception), ZEND_STRL("envelope"), PHP5to7_MAYBE_PTR(message) TSRMLS_CC);
+
+			zend_throw_exception_object(PHP5to7_MAYBE_PTR(exception) TSRMLS_CC);
+
+			PHP5to7_MAYBE_DESTROY(message);
+
+			amqp_destroy_envelope(&envelope);
+		    efree(key);
+			break;
+		}
+
+		efree(key);
 		amqp_destroy_envelope(&envelope);
 
 		/* Make the callback */
@@ -650,8 +717,9 @@ PHP_METHOD(amqp_queue_class, consume)
 		Z_ADDREF_P( PHP5to7_MAYBE_PTR(message));
 
 		/* Add a pointer to the queue: */
-		add_index_zval(PHP5to7_MAYBE_PTR(params), 1, getThis());
-		Z_ADDREF_P(getThis());
+		add_index_zval(PHP5to7_MAYBE_PTR(params), 1, PHP5to7_MAYBE_DEREF(current_queue_zv));
+		Z_ADDREF_P(PHP5to7_MAYBE_DEREF(current_queue_zv));
+
 
 		/* Convert everything to be callable */
 		zend_fcall_info_args(&fci, PHP5to7_MAYBE_PTR(params) TSRMLS_CC);
@@ -684,7 +752,7 @@ PHP_METHOD(amqp_queue_class, consume)
 /* {{{ proto int AMQPQueue::ack(long deliveryTag, [bit flags=AMQP_NOPARAM]);
 	acknowledge the message
 */
-PHP_METHOD(amqp_queue_class, ack)
+static PHP_METHOD(amqp_queue_class, ack)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -714,14 +782,10 @@ PHP_METHOD(amqp_queue_class, ack)
 		res.reply_type 	  = AMQP_RESPONSE_LIBRARY_EXCEPTION;
 		res.library_error = status;
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -733,7 +797,7 @@ PHP_METHOD(amqp_queue_class, ack)
 /* {{{ proto int AMQPQueue::nack(long deliveryTag, [bit flags=AMQP_NOPARAM]);
 	acknowledge the message
 */
-PHP_METHOD(amqp_queue_class, nack)
+static PHP_METHOD(amqp_queue_class, nack)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -764,14 +828,10 @@ PHP_METHOD(amqp_queue_class, nack)
 		res.reply_type 	  = AMQP_RESPONSE_LIBRARY_EXCEPTION;
 		res.library_error = status;
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -783,7 +843,7 @@ PHP_METHOD(amqp_queue_class, nack)
 /* {{{ proto int AMQPQueue::reject(long deliveryTag, [bit flags=AMQP_NOPARAM]);
 	acknowledge the message
 */
-PHP_METHOD(amqp_queue_class, reject)
+static PHP_METHOD(amqp_queue_class, reject)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -813,14 +873,10 @@ PHP_METHOD(amqp_queue_class, reject)
 		res.reply_type 	  = AMQP_RESPONSE_LIBRARY_EXCEPTION;
 		res.library_error = status;
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -832,7 +888,7 @@ PHP_METHOD(amqp_queue_class, reject)
 /* {{{ proto int AMQPQueue::purge();
 purge queue
 */
-PHP_METHOD(amqp_queue_class, purge)
+static PHP_METHOD(amqp_queue_class, purge)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -854,14 +910,10 @@ PHP_METHOD(amqp_queue_class, purge)
 	if (!r) {
 		amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -880,11 +932,12 @@ PHP_METHOD(amqp_queue_class, purge)
 /* {{{ proto int AMQPQueue::cancel([string consumer_tag]);
 cancel queue to consumer
 */
-PHP_METHOD(amqp_queue_class, cancel)
+static PHP_METHOD(amqp_queue_class, cancel)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
 	amqp_channel_resource *channel_resource;
+	PHP5to7_zval_t *tmp = NULL;
 
 	char *consumer_tag = NULL;  PHP5to7_param_str_len_type_t consumer_tag_len = 0;
 
@@ -892,12 +945,20 @@ PHP_METHOD(amqp_queue_class, cancel)
 		return;
 	}
 
-	channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(PHP_AMQP_READ_THIS_PROP("channel"));
+	zval *channel_zv = PHP_AMQP_READ_THIS_PROP("channel");
+	zval *consumers = zend_read_property(amqp_channel_class_entry, channel_zv, ZEND_STRL("consumers"), 0 PHP5to7_READ_PROP_RV_PARAM_CC TSRMLS_CC);
+
+	if (IS_ARRAY != Z_TYPE_P(consumers)) {
+		zend_throw_exception(amqp_queue_exception_class_entry, "Invalid channel consumers, forgot to call channel constructor?", 0 TSRMLS_CC);
+		return;
+	}
+
+	channel_resource = PHP_AMQP_GET_CHANNEL_RESOURCE(channel_zv);
 	PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not cancel queue.");
 
-    if (!consumer_tag_len && !PHP_AMQP_READ_THIS_PROP_STRLEN("consumer_tag")) {
-        return;
-    }
+	if (!consumer_tag_len && !PHP_AMQP_READ_THIS_PROP_STRLEN("consumer_tag")) {
+		return;
+	}
 
 	amqp_basic_cancel_ok_t *r = amqp_basic_cancel(
 		channel_resource->connection_resource->connection_state,
@@ -908,14 +969,10 @@ PHP_METHOD(amqp_queue_class, cancel)
 	if (!r) {
 		amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -923,7 +980,12 @@ PHP_METHOD(amqp_queue_class, cancel)
 		zend_update_property_null(this_ce, getThis(), ZEND_STRL("consumer_tag") TSRMLS_CC);
 	}
 
-	php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
+    char *key;
+    key = estrndup((char *)r->consumer_tag.bytes, (uint) r->consumer_tag.len);
+    PHP5to7_ZEND_HASH_DEL(Z_ARRVAL_P(consumers), (const char *) key, PHP5to7_ZEND_HASH_STRLEN(r->consumer_tag.len));
+    efree(key);
+
+    php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
 
 	RETURN_TRUE;
 }
@@ -933,7 +995,7 @@ PHP_METHOD(amqp_queue_class, cancel)
 /* {{{ proto int AMQPQueue::unbind(string exchangeName, [string routingKey, array arguments]);
 unbind queue from exchange
 */
-PHP_METHOD(amqp_queue_class, unbind)
+static PHP_METHOD(amqp_queue_class, unbind)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -956,7 +1018,7 @@ PHP_METHOD(amqp_queue_class, unbind)
 	PHP_AMQP_VERIFY_CHANNEL_RESOURCE(channel_resource, "Could not unbind queue.");
 
 	if (zvalArguments) {
-		arguments = convert_zval_to_amqp_table(zvalArguments TSRMLS_CC);
+		arguments = php_amqp_type_convert_zval_to_amqp_table(zvalArguments TSRMLS_CC);
 	}
 
 	amqp_queue_unbind(
@@ -969,20 +1031,14 @@ PHP_METHOD(amqp_queue_class, unbind)
 	);
 
 	if (arguments) {
-		php_amqp_free_amqp_table(arguments);
+		php_amqp_type_free_amqp_table(arguments);
 	}
 
 	amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-	if (res.reply_type != AMQP_RESPONSE_NORMAL) {
-		PHP_AMQP_INIT_ERROR_MESSAGE();
-
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+	if (PHP_AMQP_MAYBE_ERROR(res, channel_resource)) {
+		php_amqp_zend_throw_exception_short(res, amqp_queue_exception_class_entry TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -996,7 +1052,7 @@ PHP_METHOD(amqp_queue_class, unbind)
 /* {{{ proto int AMQPQueue::delete([long flags = AMQP_NOPARAM]]);
 delete queue and return the number of messages deleted in it
 */
-PHP_METHOD(amqp_queue_class, delete)
+static PHP_METHOD(amqp_queue_class, delete)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 
@@ -1024,14 +1080,10 @@ PHP_METHOD(amqp_queue_class, delete)
 	if (!r) {
 		amqp_rpc_reply_t res = amqp_get_rpc_reply(channel_resource->connection_resource->connection_state);
 
-		PHP_AMQP_INIT_ERROR_MESSAGE();
+		php_amqp_error(res, &PHP_AMQP_G(error_message), channel_resource->connection_resource, channel_resource TSRMLS_CC);
 
-		php_amqp_error(res, PHP_AMQP_ERROR_MESSAGE_PTR, channel_resource->connection_resource, channel_resource TSRMLS_CC);
-
-		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_ERROR_MESSAGE, 0 TSRMLS_CC);
+		php_amqp_zend_throw_exception(res, amqp_queue_exception_class_entry, PHP_AMQP_G(error_message), PHP_AMQP_G(error_code) TSRMLS_CC);
 		php_amqp_maybe_release_buffers_on_channel(channel_resource->connection_resource, channel_resource);
-
-		PHP_AMQP_DESTROY_ERROR_MESSAGE();
 		return;
 	}
 
@@ -1045,7 +1097,7 @@ PHP_METHOD(amqp_queue_class, delete)
 
 /* {{{ proto AMQPChannel::getChannel()
 Get the AMQPChannel object in use */
-PHP_METHOD(amqp_queue_class, getChannel)
+static PHP_METHOD(amqp_queue_class, getChannel)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -1055,7 +1107,7 @@ PHP_METHOD(amqp_queue_class, getChannel)
 
 /* {{{ proto AMQPChannel::getConnection()
 Get the AMQPConnection object in use */
-PHP_METHOD(amqp_queue_class, getConnection)
+static PHP_METHOD(amqp_queue_class, getConnection)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -1065,7 +1117,7 @@ PHP_METHOD(amqp_queue_class, getConnection)
 
 /* {{{ proto string AMQPChannel::getConsumerTag()
 Get latest consumer tag*/
-PHP_METHOD(amqp_queue_class, getConsumerTag)
+static PHP_METHOD(amqp_queue_class, getConsumerTag)
 {
 	PHP5to7_READ_PROP_RV_PARAM_DECL;
 	PHP_AMQP_NOPARAMS();
@@ -1207,7 +1259,7 @@ zend_function_entry amqp_queue_class_functions[] = {
 
 		PHP_MALIAS(amqp_queue_class, declare, declareQueue, arginfo_amqp_queue_class_declareQueue,	ZEND_ACC_PUBLIC | ZEND_ACC_DEPRECATED)
 
-		{NULL, NULL, NULL}	/* Must be the last line in amqp_functions[] */
+		{NULL, NULL, NULL}
 };
 
 PHP_MINIT_FUNCTION(amqp_queue)
